@@ -42,6 +42,7 @@
 #ifdef WITH_MOZLDAP
 #include <mozldap/ldap.h>
 #else
+#define LDAP_DEPRECATED 1
 #include <ldap.h>
 #endif
 #include <sasl/sasl.h>
@@ -481,7 +482,7 @@ int ldap_pwd_change(char *client_name, char *realm_name, krb5_data pwd, char **e
 		goto done;
 	}
 
-	ber_printf(ctrl, "{tstON}",
+	ber_printf(ctrl, "{tstO}",
 		   LDAP_TAG_EXOP_MODIFY_PASSWD_ID, userdn,
 		   LDAP_TAG_EXOP_MODIFY_PASSWD_NEW, &newpw);
 
@@ -1004,7 +1005,9 @@ pid_t handle_conn(struct socklist *sd)
 	int ret;
 
 	fromlen = sizeof(from);
+	mfd = 0;
 	tcp = 0;
+	reqlen = 0;
 
 	/* receive request */
 	if (sd->socktype == SOCK_STREAM) {
@@ -1296,7 +1299,11 @@ int main(int argc, char *argv[])
 
 	for (tifa = ifa; tifa; tifa = tifa->ifa_next) {
 
-		if (tifa->ifa_addr->sa_family != PF_INET &&
+		if (NULL == tifa->ifa_addr)
+			/* uhmm no address ?? skip it */
+			continue;
+
+		if (tifa->ifa_addr->sa_family != AF_INET &&
 		    tifa->ifa_addr->sa_family != AF_INET6) {
 			/* not interesting for us */
 			continue;
@@ -1305,7 +1312,7 @@ int main(int argc, char *argv[])
 		ret = getnameinfo(tifa->ifa_addr, sizeof(struct sockaddr_storage),
 				  host, sizeof(host), NULL, 0, NI_NUMERICHOST);
 		if (ret) {
-			syslog(LOG_ERR, "Error converting address (%d)",
+			syslog(LOG_ERR, "Error converting address (%s)",
 				gai_strerror(ret));
 			continue;
 		} else {
@@ -1316,10 +1323,10 @@ int main(int argc, char *argv[])
 		hints.ai_flags = AI_NUMERICHOST;
 		hints.ai_family = AF_UNSPEC;
 
-		/* this shoud return 2 entries, one for UDP and one for TCP */
+		/* this should return 2 entries, one for UDP and one for TCP */
 		ret = getaddrinfo(host, "kpasswd", &hints, &ai);
 		if (ret) {
-			syslog(LOG_ERR, "Error getting address info (%d) for [%s]",
+			syslog(LOG_ERR, "Error getting address info (%s) for [%s]",
 				gai_strerror(ret), host);
 			continue;
 		}
